<?php

namespace Drupal\search_api_decoupled_ui\Plugin\Block;

use Drupal\Core\Block\BlockBase;
use Drupal\Core\Cache\CacheableMetadata;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Layout\LayoutPluginManagerInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a block with decoupled search.
 *
 * @Block(
 *   id = "search_api_endpoint",
 *   admin_label = @Translation("Search API Endpoint"),
 *   category = @Translation("Search API Decoupled"),
 *   deriver = "Drupal\search_api_decoupled_ui\Plugin\Derivative\SearchApiEndpointBlock",
 * )
 */
class SearchApiEndpointBlock extends BlockBase implements ContainerFactoryPluginInterface {

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The layout plugin manager.
   *
   * @var \Drupal\Core\Layout\LayoutPluginManagerInterface
   */
  protected $layoutPluginManager;

  /**
   * The module handler.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected $moduleHandler;

  /**
   * Constructs a new SearchBlock object.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\Core\Layout\LayoutPluginManagerInterface $layout_plugin_manager
   *   The layout plugin manager.
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $module_handler
   *   The module handler.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, EntityTypeManagerInterface $entity_type_manager, LayoutPluginManagerInterface $layout_plugin_manager, ModuleHandlerInterface $module_handler) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->entityTypeManager = $entity_type_manager;
    $this->layoutPluginManager = $layout_plugin_manager;
    $this->moduleHandler = $module_handler;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('entity_type.manager'),
      $container->get('plugin.manager.core.layout'),
      $container->get('module_handler')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function build() {
    /** @var \Drupal\search_api_decoupled_ui\SearchApiEndpointUiInterface $search_api_endpoint */
    $search_api_endpoint = $this->entityTypeManager->getStorage('search_api_endpoint')->load($this->getDerivativeId());
    $content = [];
    $layout = $this->layoutPluginManager->getDefinition($search_api_endpoint->getLayout());
    $layout_regions = $layout->getRegions();
    foreach (array_keys($search_api_endpoint->getUiElementRegions()) as $region) {
      $content[$region] = ['#markup' => '<p class="visually-hidden">' . $this->t('Region @region', ['@region' => $layout_regions[$region]['label']]) . '</p>'];
    }
    $cacheable_metadata = new CacheableMetadata();
    $block_id = 'block-' . $this->getMachineNameSuggestion();
    $build = $content + [
      '#theme' => $layout->getThemeHook(),
      '#layout' => $layout,
      '#attached' => [
        'library' => [
          'search_api_decoupled_ui/client',
          $layout->getLibrary(),
        ],
        'drupalSettings' => [
          'search_api_endpoint' => [
            $block_id => $search_api_endpoint->getUiSettings($cacheable_metadata),
          ],
        ],
      ],
    ];
    $this->moduleHandler->alter('sae_ui_block', $build, $search_api_endpoint, $cacheable_metadata);
    $cacheable_metadata->applyTo($build);
    return $build;
  }

}
